#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the PostgreSQL application log file text parser plugin."""

import unittest

from plaso.parsers.text_plugins import postgresql

from tests.parsers.text_plugins import test_lib


class PostgreSQLTextPluginTest(test_lib.TextPluginTestCase):
  """The PostgreSQL application log file text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = postgresql.PostgreSQLTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['postgresql.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 20)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test a regular log entry.
    expected_event_values = {
        'data_type': 'postgresql:application_log:entry',
        'log_line': (
            'starting PostgreSQL 12.9 (Ubuntu 12.9-0ubuntu0.20.04.1) '
            'on x86_64-pc-linux-gnu, compiled by gcc '
            '(Ubuntu 9.3.0-17ubuntu1~20.04) 9.3.0, 64-bit'),
        'pid': '7755',
        'recorded_time': '2022-04-12T00:16:05.526+00:00',
        'severity': 'LOG'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test a log entry containing user and database.
    expected_event_values = {
        'data_type': 'postgresql:application_log:entry',
        'log_line': 'password authentication failed for user "postgres"',
        'pid': '9158',
        'recorded_time': '2022-04-12T00:24:24.741+00:00',
        'severity': 'FATAL',
        'user': 'postgres@postgres'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 15)
    self.CheckEventData(event_data, expected_event_values)

    # Check a multi-line log entry.
    expected_event_values = {
        'data_type': 'postgresql:application_log:entry',
        'log_line': (
            'User "postgres" has no password assigned.\n        '
            'Connection matched pg_hba.conf line 96: '
            '"host    all             all             '
            '127.0.0.1/32            md5"'),
        'pid': '9158',
        'recorded_time': '2022-04-12T00:24:24.741+00:00',
        'severity': 'DETAIL'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 16)
    self.CheckEventData(event_data, expected_event_values)

    # Check a log entry with a time zone.
    expected_event_values = {
        'data_type': 'postgresql:application_log:entry',
        'log_line': (
            'could not receive data from client: Connection reset by peer'),
        'pid': '203851-1',
        'recorded_time': '2022-07-15T23:04:27',
        'severity': 'LOG'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 19)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OpenSearch output module CLI arguments helper."""

import argparse
import unittest

from plaso.cli.helpers import opensearch_output
from plaso.lib import errors
from plaso.output import opensearch

from tests import test_lib as shared_test_lib
from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class OpenSearchOutputArgumentsHelperTest(
    test_lib.OutputModuleArgumentsHelperTest):
  """Tests the OpenSearch output module CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--index_name NAME] [--flush_interval INTERVAL]
                     [--opensearch-server HOSTNAME] [--opensearch-port PORT]
                     [--opensearch-user USERNAME]
                     [--opensearch-password PASSWORD]
                     [--opensearch-mappings PATH]
                     [--opensearch-url-prefix URL_PREFIX] [--use_ssl]
                     [--ca_certificates_file_path PATH]

Test argument parser.

{0:s}:
  --ca_certificates_file_path PATH, --ca-certificates-file-path PATH
                        Path to a file containing a list of root certificates
                        to trust.
  --flush_interval INTERVAL, --flush-interval INTERVAL
                        Events to queue up before bulk insert to OpenSearch.
  --index_name NAME, --index-name NAME
                        Name of the index in OpenSearch.
  --opensearch-mappings PATH, --opensearch_mappings PATH
                        Path to a file containing mappings for OpenSearch
                        indexing.
  --opensearch-password PASSWORD, --opensearch_password PASSWORD
                        Password to use for OpenSearch authentication.
                        WARNING: use with caution since this can expose the
                        password to other users on the system. The password
                        can also be set with the environment variable
                        PLASO_OPENSEARCH_PASSWORD.
  --opensearch-port PORT, --opensearch_port PORT, --port PORT
                        Port number of the OpenSearch server.
  --opensearch-server HOSTNAME, --opensearch_server HOSTNAME, --server HOSTNAME
                        Hostname or IP address of the OpenSearch server.
  --opensearch-url-prefix URL_PREFIX, --opensearch_url_prefix URL_PREFIX
                        URL prefix for OpenSearch.
  --opensearch-user USERNAME, --opensearch_user USERNAME
                        Username to use for OpenSearch authentication.
  --use_ssl, --use-ssl  Enforces use of SSL/TLS.
""".format(cli_test_lib.ARGPARSE_OPTIONS)

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py',
        description='Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    opensearch_output.OpenSearchOutputArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options._data_location = shared_test_lib.DATA_PATH

    output_module = opensearch.OpenSearchOutputModule()
    opensearch_output.OpenSearchOutputArgumentsHelper.ParseOptions(
        options, output_module)

    with self.assertRaises(errors.BadConfigObject):
      opensearch_output.OpenSearchOutputArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the vsftpd log file text parser plugin."""

import unittest

from plaso.parsers.text_plugins import vsftpd

from tests.parsers.text_plugins import test_lib


class VsftpdLogTextPluginText(test_lib.TextPluginTestCase):
  """Tests for the vsftpd log file text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = vsftpd.VsftpdLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['vsftpd.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 25)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2016-06-10T14:24:19',
        'data_type': 'vsftpd:log',
        'text': (
            '[pid 3] [jean] OK DOWNLOAD: Client "192.168.1.7", '
            '"/home/jean/trains/how-thomas-the-tank-engine-works-1.jpg", '
            '49283 bytes, 931.38Kbyte/sec')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 12)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Interface for SQLite database file parser plugins."""

import sqlite3

from dfdatetime import filetime as dfdatetime_filetime
from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.parsers import logger
from plaso.parsers import plugins


class SQLitePlugin(plugins.BasePlugin):
  """SQLite parser plugin."""

  NAME = 'sqlite_plugin'
  DATA_FORMAT = 'SQLite database file'

  # Dictionary of frozensets containing the columns in tables that must be
  # present in the database for the plugin to run.
  # This generally should only include tables/columns that are used in SQL
  # queries by the plugin and not include extraneous tables/columns to better
  # accommodate future application database versions. The exception to this is
  # when extra tables/columns are needed to identify the target database from
  # others with a similar structure.
  REQUIRED_STRUCTURE = {}

  # Queries to be executed.
  # Should be a list of tuples with two entries, SQLCommand and callback
  # function name.
  QUERIES = []

  # Database schemas this plugin was originally designed for.
  # Should be a list of dictionaries with {table_name: SQLCommand} format.
  SCHEMAS = []

  # Value to indicate the schema of the database must match one of the schemas
  # defined by the plugin.
  REQUIRES_SCHEMA_MATCH = False

  def __init__(self):
    """Initializes a SQLite parser plugin."""
    super(SQLitePlugin, self).__init__()
    self._keys_per_query = {}

  def _GetDateTimeStringRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time string value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.TimeElements: date and time value or None if not available.
    """
    date_time_string = self._GetRowValue(query_hash, row, value_name)
    if date_time_string is None:
      return None

    date_time = dfdatetime_time_elements.TimeElements()
    date_time.CopyFromDateTimeString(date_time_string)
    return date_time

  def _GeFiletimeRowValue(self, query_hash, row, value_name):
    """Retrieves a FILETIME date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.Filetime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_filetime.Filetime(timestamp=timestamp)

  def _GetRowValue(self, query_hash, row, value_name):
    """Retrieves a value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      object: value or None if not available.
    """
    keys_name_to_index_map = self._keys_per_query.get(query_hash, None)
    if not keys_name_to_index_map:
      keys_name_to_index_map = {
          name: index for index, name in enumerate(row.keys())}
      self._keys_per_query[query_hash] = keys_name_to_index_map

    value_index = keys_name_to_index_map.get(value_name)

    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".
    return row[value_index]

  @classmethod
  def _HashRow(cls, row):
    """Hashes the given row.

    Args:
      row (sqlite3.Row): row.

    Returns:
      int: hash value of the given row.
    """
    values = []
    for value in row:
      try:
        value = '{0!s}'.format(value)
      except UnicodeDecodeError:
        # In Python 2, blobs are "read-write buffer" and will cause a
        # UnicodeDecodeError exception if we try format it as a string.
        # Since Python 3 does not support the buffer type we cannot check
        # the type of value.
        value = repr(value)

      values.append(value)

    return hash(' '.join(values))

  def _ParseSQLiteDatabase(
      self, parser_mediator, database, query, callback, cache):
    """Extracts events from a SQLite database.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      database (SQLiteDatabase): database.
      query (str): query.
      callback (function): function to invoke to parse an individual row.
      cache (SQLiteCache): cache.
    """
    row_cache = cache.GetRowCache(query)

    try:
      rows = database.Query(query)

    except sqlite3.DatabaseError as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to run query: {0:s} on database with error: {1!s}'.format(
              query, exception))
      return

    for index, row in enumerate(rows):
      if parser_mediator.abort:
        break

      row_hash = self._HashRow(row)
      if row_hash in row_cache:
        continue

      try:
        callback(parser_mediator, query, row, cache=cache, database=database)

      except Exception as exception:  # pylint: disable=broad-except
        parser_mediator.ProduceExtractionWarning((
            'unable to parse row: {0:d} with callback: {1:s} on database '
            'with error: {2!s}').format(
                index, callback.__name__, exception))
        # TODO: consider removing return.
        return

      row_cache.add(row_hash)

  def CheckRequiredTablesAndColumns(self, database):
    """Check if the database has the minimal structure required by the plugin.

    Args:
      database (SQLiteDatabase): the database who's structure is being checked.

    Returns:
      bool: True if the database has the required tables and columns defined by
          the plugin, or False if it does not or if the plugin does not define
          required tables and columns. The database can have more tables and/or
          columns than specified by the plugin and still return True.
    """
    if not self.REQUIRED_STRUCTURE:
      return False

    has_required_structure = True
    for required_table, required_columns in self.REQUIRED_STRUCTURE.items():
      if required_table not in database.tables:
        has_required_structure = False
        break

      if not frozenset(required_columns).issubset(
          database.columns_per_table.get(required_table)):
        has_required_structure = False
        break

    return has_required_structure

  def CheckSchema(self, database):
    """Checks the schema of a database with that defined in the plugin.

    Args:
      database (SQLiteDatabase): SQLite database to check.

    Returns:
      bool: True if the schema of the database matches that defined by
          the plugin, or False if the schemas do not match or no schema
          is defined by the plugin.
    """
    schema_match = False
    if self.SCHEMAS:
      for schema in self.SCHEMAS:
        if database and database.schema == schema:
          schema_match = True

    return schema_match

  # pylint: disable=arguments-differ
  def Process(
      self, parser_mediator, cache=None, database=None, **unused_kwargs):
    """Extracts events from a SQLite database.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      cache (Optional[SQLiteCache]): cache.
      database (Optional[SQLiteDatabase]): database.

    Raises:
      ValueError: If the database or cache value are missing.
    """
    if cache is None:
      raise ValueError('Missing cache value.')

    if database is None:
      raise ValueError('Missing database value.')

    # This will raise if unhandled keyword arguments are passed.
    super(SQLitePlugin, self).Process(parser_mediator)

    for query, callback_method in self.QUERIES:
      if parser_mediator.abort:
        break

      callback = getattr(self, callback_method, None)
      if callback is None:
        logger.warning(
            '[{0:s}] missing callback method: {1:s} for query: {2:s}'.format(
                self.NAME, callback_method, query))
        continue

      self._ParseSQLiteDatabase(
          parser_mediator, database, query, callback, cache)



# -*- coding: utf-8 -*-
"""The cookie plugins helper mix-in."""

# Register the cookie plugins.
from plaso.parsers import cookie_plugins  # pylint: disable=unused-import
from plaso.parsers.cookie_plugins import manager as cookie_plugins_manager


class CookiePluginsHelper(object):
  """Cookie plugins helper mix-in."""

  def __init__(self):
    """Initializes the cookie plugins helper mix-in."""
    super(CookiePluginsHelper, self).__init__()
    self._cookie_plugins = (
        cookie_plugins_manager.CookiePluginsManager.GetPlugins())

  def _ParseCookie(self, parser_mediator, cookie_name, cookie_data, url):
    """Parses a cookie.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      cookie_name (str): the name of the cookie value.
      cookie_data (bytes): the cookie data, as a byte sequence.
      url (str): the full URL or path where the cookie was set.
    """
    for cookie_plugin in self._cookie_plugins:
      if parser_mediator.abort:
        break

      if cookie_name == cookie_plugin.COOKIE_NAME:
        try:
          cookie_plugin.UpdateChainAndProcess(
              parser_mediator, cookie_data=cookie_data, cookie_name=cookie_name,
              url=url)

        except Exception as exception:  # pylint: disable=broad-except
          parser_mediator.ProduceExtractionWarning(
              'plugin: {0:s} unable to parse cookie with error: {1!s}'.format(
                  cookie_plugin.NAME, exception))



# -*- coding: utf-8 -*-
"""Parser for Extensible Storage Engine (ESE) database files (EDB)."""

import pyesedb

from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import logger
from plaso.parsers import manager
from plaso.parsers import plugins


class ESEDatabase(object):
  """Extensible Storage Engine (ESE) database."""

  def __init__(self):
    """Initializes an Extensible Storage Engine (ESE) database."""
    super(ESEDatabase, self).__init__()
    self._esedb_file = None
    self._table_names = []

  @property
  def tables(self):
    """List[str]: names of all the tables."""
    if not self._table_names:
      for esedb_table in self._esedb_file.tables:
        self._table_names.append(esedb_table.name)

    return self._table_names

  def Close(self):
    """Closes the database."""
    self._esedb_file.close()
    self._esedb_file = None

  def GetTableByName(self, name):
    """Retrieves a table by its name.

    Args:
      name (str): name of the table.

    Returns:
      pyesedb.table: the table with the corresponding name or None if there is
          no table with the name.
    """
    return self._esedb_file.get_table_by_name(name)

  def Open(self, file_object):
    """Opens an Extensible Storage Engine (ESE) database file.

    Args:
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      IOError: if the file-like object cannot be read.
      OSError: if the file-like object cannot be read.
      ValueError: if the file-like object is missing.
    """
    if not file_object:
      raise ValueError('Missing file object.')

    self._esedb_file = pyesedb.file()
    self._esedb_file.open_file_object(file_object)


class ESEDBCache(plugins.BasePluginCache):
  """A cache storing query results for ESEDB plugins."""

  def StoreDictInCache(self, attribute_name, dict_object):
    """Store a dict object in cache.

    Args:
      attribute_name (str): name of the attribute.
      dict_object (dict): dictionary.
    """
    setattr(self, attribute_name, dict_object)


class ESEDBParser(interface.FileObjectParser):
  """Parses Extensible Storage Engine (ESE) database files (EDB)."""

  _INITIAL_FILE_OFFSET = None

  NAME = 'esedb'
  DATA_FORMAT = 'Extensible Storage Engine (ESE) Database File (EDB) format'

  _plugin_classes = {}

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'\xef\xcd\xab\x89', offset=4)
    return format_specification

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses an ESE database file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
    """
    database = ESEDatabase()

    try:
      database.Open(file_object)
    except (IOError, ValueError) as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to open file with error: {0!s}'.format(exception))
      return

    display_name = parser_mediator.GetDisplayName()

    # Compare the list of available plugin objects.
    cache = ESEDBCache()
    try:
      for plugin_name, plugin in self._plugins_per_name.items():
        if parser_mediator.abort:
          break

        profiling_name = '/'.join([self.NAME, plugin.NAME])

        parser_mediator.SampleFormatCheckStartTiming(profiling_name)

        try:
          result = plugin.CheckRequiredTables(database)
        finally:
          parser_mediator.SampleFormatCheckStopTiming(profiling_name)

        if not result:
          logger.debug('Skipped parsing file: {0:s} with plugin: {1:s}'.format(
              display_name, plugin_name))
          continue

        logger.debug('Parsing file: {0:s} with plugin: {1:s}'.format(
            display_name, plugin_name))

        parser_mediator.SampleStartTiming(profiling_name)

        try:
          plugin.UpdateChainAndProcess(
              parser_mediator, cache=cache, database=database)

        except Exception as exception:  # pylint: disable=broad-except
          parser_mediator.ProduceExtractionWarning((
              'plugin: {0:s} unable to parse ESE database with error: '
              '{1!s}').format(plugin_name, exception))

        finally:
          parser_mediator.SampleStopTiming(profiling_name)

    finally:
      # TODO: explicitly clean up cache.

      database.Close()


manager.ParsersManager.RegisterParser(ESEDBParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows Registry file parser."""

import unittest

from artifacts import reader as artifacts_reader
from artifacts import registry as artifacts_registry

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import events
from plaso.engine import artifact_filters
from plaso.parsers import mediator as parsers_mediator
from plaso.parsers import winreg_parser
# Register all plugins.
from plaso.parsers import winreg_plugins  # pylint: disable=unused-import

from tests.parsers import test_lib


class WinRegistryParserTest(test_lib.ParserTestCase):
  """Tests for the Windows Registry file parser."""

  # pylint: disable=protected-access

  def _GetParserChainOfPlugin(self, plugin_name):
    """Determines the parser chain of a parser plugin.

    Args:
      plugin_name (str): name of the parser plugin.

    Return:
      str: parser chain of the parser plugin.
    """
    return 'winreg/{0:s}'.format(plugin_name)

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = winreg_parser.WinRegistryParser()

    number_of_plugins = len(parser._plugin_classes)

    parser.EnablePlugins([])
    self.assertEqual(len(parser._plugins_per_name), 0)

    parser.EnablePlugins(parser.ALL_PLUGINS)
    # Extract 1 for the default plugin.
    self.assertEqual(len(parser._plugins_per_name), number_of_plugins - 1)

    parser.EnablePlugins(['appcompatcache'])
    self.assertEqual(len(parser._plugins_per_name), 1)

  def testParse(self):
    """Test the parse function on a Windows NT Registry file."""
    parser = winreg_parser.WinRegistryParser()
    storage_writer = self._ParseFile([
        'regf', '100_sub_keys.hiv'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 101)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseCorruptionInSubKeyList(self):
    """Test the parse function on a corrupted Windows NT Registry file."""
    parser = winreg_parser.WinRegistryParser()
    storage_writer = self._ParseFile([
        'regf', 'corrupt_sub_key_list.hiv'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 100)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseNTUserDat(self):
    """Tests the Parse function on a NTUSER.DAT file."""
    parser = winreg_parser.WinRegistryParser()
    parser.EnablePlugins(parser.ALL_PLUGINS)
    storage_writer = self._ParseFile(['NTUSER.DAT'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1190)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseNoRootKey(self):
    """Test the parse function on a Registry file with no root key."""
    parser = winreg_parser.WinRegistryParser()
    storage_writer = self._ParseFile(['ntuser.dat.LOG'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseSystem(self):
    """Tests the Parse function on a SYSTEM file."""
    parser = winreg_parser.WinRegistryParser()
    parser.EnablePlugins(parser.ALL_PLUGINS)
    storage_writer = self._ParseFile(['SYSTEM'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 31432)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseSystemWithArtifactFilters(self):
    """Tests the Parse function on a SYSTEM file with artifact filters."""
    artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(artifacts_path)

    parser = winreg_parser.WinRegistryParser()
    parser.EnablePlugins(parser.ALL_PLUGINS)

    artifact_filter_names = ['TestRegistryKey', 'TestRegistryValue']
    registry = artifacts_registry.ArtifactDefinitionsRegistry()
    reader = artifacts_reader.YamlArtifactsReader()

    registry.ReadFromDirectory(reader, artifacts_path)

    artifacts_filters_helper = (
        artifact_filters.ArtifactDefinitionsFiltersHelper(registry))

    artifacts_filters_helper.BuildFindSpecs(artifact_filter_names)

    storage_writer = self._ParseFile(
        ['SYSTEM'], parser,
        registry_find_specs=artifacts_filters_helper.registry_find_specs)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3535)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseSystemWithBinaryValues(self):
    """Tests the Parse function on a SYSTEM file with binary values."""
    test_file_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_file_path)

    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)

    parser = winreg_parser.WinRegistryParser()
    parser.EnablePlugins(parser.ALL_PLUGINS)

    parser_mediator = parsers_mediator.ParserMediator()

    storage_writer = self._CreateStorageWriter()
    parser_mediator.SetStorageWriter(storage_writer)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(path_spec)
    parser_mediator.SetFileEntry(file_entry)

    parser_mediator.SetExtractWinRegBinaryValues(True)

    event_data_stream = events.EventDataStream()
    event_data_stream.path_spec = file_entry.path_spec
    parser_mediator.ProduceEventDataStream(event_data_stream)

    file_object = file_entry.GetFileObject()
    parser.Parse(parser_mediator, file_object)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 31432)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*
"""Tests for the JSON-L parser plugin for Google Cloud (GCP) log files."""

import unittest

from plaso.parsers.jsonl_plugins import gcp_log

from tests.parsers.jsonl_plugins import test_lib


class GCPLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Google Cloud (GCP) log files."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = gcp_log.GCPLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['gcp_logging.jsonl'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'container': None,
        'event_subtype': 'beta.compute.networks.insert',
        'event_type': None,
        'filename': None,
        'firewall_rules': None,
        'firewall_source_ranges': None,
        'log_name': (
            'projects/fake-project/logs/cloudaudit.googleapis.com%2Factivity'),
        'message': None,
        'policy_deltas': None,
        'recorded_time': '2021-10-19T02:57:47.339377+00:00',
        'request_account_identifier': None,
        'request_description': None,
        'request_direction': None,
        'request_email': None,
        'request_member': None,
        'request_metadata': [
            'callerIp: 1.1.1.1', 'callerSuppliedUserAgent: UserAgent'],
        'request_name': None,
        'request_target_tags': None,
        'resource_labels': [
            'network_id: 4881930676783516364', 'project_id: fake-project'],
        'resource_name': 'projects/fake-project/global/networks/test',
        'service_account_display_name': None,
        'service_name': 'compute.googleapis.com',
        'severity': 'NOTICE',
        'text_payload': None,
        'user': 'fakeemailxyz@gmail.com'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Plist parser plugin for Mac OS login items plist files."""

import os

from dfdatetime import hfs_time as dfdatetime_hfs_time

from dtfabric.runtime import data_maps as dtfabric_data_maps

from plaso.containers import events
from plaso.lib import dtfabric_helper
from plaso.lib import errors
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class MacOSLoginItemEventData(events.EventData):
  """Mac OS login item event data.

  Attributes:
    hidden (bool): whether this login item is hidden.
    name (str): name.
    target_creation_time (dfdatetime.DateTimeValues): date and time the target
        was created.
    target_path (str): path of the target.
    volume_creation_time (dfdatetime.DateTimeValues): date and time the (target)
        volume was created.
    volume_flags (int): volume flags.
    volume_mount_point (str): location the volume is mounted on the file system.
    volume_name (str): name of the volume containing the target.
  """

  DATA_TYPE = 'macos:login_items:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSLoginItemEventData, self).__init__(data_type=self.DATA_TYPE)
    self.hidden = None
    self.name = None
    self.target_creation_time = None
    self.target_path = None
    self.volume_creation_time = None
    self.volume_flags = None
    self.volume_mount_point = None
    self.volume_name = None


class MacOSLoginItemsPlistPlugin(
    interface.PlistPlugin, dtfabric_helper.DtFabricHelper):
  """Plist parser plugin for Mac OS login items."""

  NAME = 'macos_login_items_plist'
  DATA_FORMAT = 'Mac OS com.apple.loginitems.plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('com.apple.loginitems.plist')])

  PLIST_KEYS = frozenset(['SessionItems'])

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'alias_data.yaml')

  def _ParseAliasData(self, alias_data, event_data):
    """Parses alias data.

    Args:
      alias_data (bytes): alias data.
      event_data (MacOSLoginItemEventData): event data.

    Raises:
      ParseError: if the value cannot be parsed.
    """
    data_type_map = self._GetDataTypeMap('alias_data_record_header')

    record_header = self._ReadStructureFromByteStream(
        alias_data, 0, data_type_map)
    data_offset = 8

    if record_header.application_information != b'\x00\x00\x00\x00':
      raise errors.ParseError('Unsupported alias application information')

    if record_header.record_size != len(alias_data):
      raise errors.ParseError('Unsupported alias record size')

    # TODO: add format version 2 support, but need test data.
    data_type_map = self._GetDataTypeMap('alias_data_record_v3')

    record_data = self._ReadStructureFromByteStream(
        alias_data[data_offset:], data_offset, data_type_map)
    data_offset += 50

    hfs_timestamp, _ = divmod(record_data.target_creation_time, 65536)
    event_data.target_creation_time = dfdatetime_hfs_time.HFSTime(
        timestamp=hfs_timestamp)

    event_data.volume_flags = record_data.volume_flags

    hfs_timestamp, _ = divmod(record_data.volume_creation_time, 65536)
    event_data.volume_creation_time = dfdatetime_hfs_time.HFSTime(
        timestamp=hfs_timestamp)

    relative_target_path = None

    while data_offset < record_header.record_size:
      data_type_map = self._GetDataTypeMap('alias_data_tagged_value')

      context = dtfabric_data_maps.DataTypeMapContext()

      tagged_value = self._ReadStructureFromByteStream(
          alias_data[data_offset:], data_offset, data_type_map,
          context=context)
      data_offset += context.byte_size

      if tagged_value.value_tag == 0xffff:
        break

      if tagged_value.value_tag == 0x000f:
        event_data.volume_name = tagged_value.string

      elif tagged_value.value_tag == 0x0012:
        relative_target_path = tagged_value.string

      elif tagged_value.value_tag == 0x0013:
        volume_mount_point = tagged_value.string
        if relative_target_path:
          relative_target_path = ''.join([
              volume_mount_point, relative_target_path])

        event_data.volume_mount_point = volume_mount_point

    if relative_target_path:
      event_data.target_path = relative_target_path

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, top_level=None, **unused_kwargs):
    """Extracts login item information from the plist.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      top_level (Optional[dict[str, object]]): plist top-level item.
    """
    session_items = top_level.get('SessionItems')
    if not session_items:
      return

    for custom_list_item in session_items.get('CustomListItems'):
      alias_data = custom_list_item.get('Alias')
      properties = custom_list_item.get('CustomItemProperties', {})

      event_data = MacOSLoginItemEventData()
      event_data.name = custom_list_item.get('Name')
      event_data.hidden = properties.get(
          'com.apple.LSSharedFileList.ItemIsHidden', False)

      if alias_data is not None:
        self._ParseAliasData(alias_data, event_data)

      parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(MacOSLoginItemsPlistPlugin)



# -*- coding: utf-8 -*-
"""Text parser plugin for MacOS Wi-Fi log (wifi.log) files."""

import re

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import dateless_helper
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class MacOSWiFiLogEventData(events.EventData):
  """MacOS Wi-Fi log event data.

  Attributes:
    action (str): known Wi-Fi action, for example connected to an access point,
        configured, etc. If the action is not known, the value is the message
        of the log (text variable).
    added_time (dfdatetime.DateTimeValues): date and time the log entry
        was added.
    agent (str): name and identifier of process that generated the log message.
    function (str): name of function that generated the log message.
    text (str): log message.
  """

  DATA_TYPE = 'macos:wifi_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSWiFiLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.action = None
    self.added_time = None
    self.agent = None
    self.function = None
    self.text = None


class MacOSWiFiLogTextPlugin(
    interface.TextPlugin, dateless_helper.DateLessLogFormatHelper):
  """Text parser plugin MacOS Wi-Fi log (wifi.log) files."""

  NAME = 'mac_wifi'
  DATA_FORMAT = 'MacOS Wi-Fi log (wifi.log) file'

  ENCODING = 'utf-8'

  _ONE_OR_TWO_DIGITS = pyparsing.Word(pyparsing.nums, max=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_DIGITS = pyparsing.Word(pyparsing.nums, exact=3).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_LETTERS = pyparsing.Word(pyparsing.alphas, exact=3)

  # Regular expressions for known actions.
  _CONNECTED_RE = re.compile(r'Already\sassociated\sto\s(.*)\.\sBailing')
  _WIFI_PARAMETERS_RE = re.compile(
      r'\[ssid=(.*?), bssid=(.*?), security=(.*?), rssi=')

  _KNOWN_FUNCTIONS = [
      'airportdProcessDLILEvent',
      '_doAutoJoin',
      '_processSystemPSKAssoc']

  _AGENT = (
      pyparsing.Literal('<') +
      pyparsing.Combine(
          pyparsing.Literal('airportd') + pyparsing.CharsNotIn('>'),
          joinString='', adjacent=True).set_results_name('agent') +
      pyparsing.Literal('>'))

  _DATE_TIME = pyparsing.Group(
      _THREE_LETTERS.set_results_name('weekday') +
      _THREE_LETTERS.set_results_name('month') +
      _ONE_OR_TWO_DIGITS.set_results_name('day_of_month') +
      _TWO_DIGITS.set_results_name('hours') + pyparsing.Suppress(':') +
      _TWO_DIGITS.set_results_name('minutes') + pyparsing.Suppress(':') +
      _TWO_DIGITS.set_results_name('seconds') + pyparsing.Suppress('.') +
      _THREE_DIGITS.set_results_name('milliseconds'))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  # Log line with a known function name.
  _KNOWN_FUNCTION_LOG_LINE = (
      _DATE_TIME.set_results_name('date_time') + _AGENT +
      pyparsing.one_of(_KNOWN_FUNCTIONS).set_results_name('function') +
      pyparsing.Literal(':') +
      pyparsing.restOfLine().set_results_name('text') +
      _END_OF_LINE)

  # Log line with an unknown function name.
  _LOG_LINE = (
      _DATE_TIME.set_results_name('date_time') + pyparsing.NotAny(
          _AGENT +
          pyparsing.one_of(_KNOWN_FUNCTIONS) +
          pyparsing.Literal(':')) +
      pyparsing.restOfLine().set_results_name('text') +
      _END_OF_LINE)

  _HEADER_LOG_LINE = (
      _DATE_TIME.set_results_name('date_time') +
      pyparsing.Literal('***Starting Up***').set_results_name('text') +
      _END_OF_LINE)

  _DATE_TIME_TURNED_OVER_HEADER = pyparsing.Group(
      _THREE_LETTERS.set_results_name('month') +
      _ONE_OR_TWO_DIGITS.set_results_name('day_of_month') +
      _TWO_DIGITS.set_results_name('hours') + pyparsing.Suppress(':') +
      _TWO_DIGITS.set_results_name('minutes') + pyparsing.Suppress(':') +
      _TWO_DIGITS.set_results_name('seconds'))

  _TURNED_OVER_HEADER_LOG_LINE = (
      _DATE_TIME_TURNED_OVER_HEADER.set_results_name('date_time') +
      pyparsing.Combine(
          pyparsing.Word(pyparsing.printables) +
          pyparsing.Word(pyparsing.printables) +
          pyparsing.Literal('logfile turned over'),
          joinString=' ', adjacent=False).set_results_name('text') +
      _END_OF_LINE)

  _LINE_STRUCTURES = [
      ('header_line', _HEADER_LOG_LINE),
      ('known_function_log_line', _KNOWN_FUNCTION_LOG_LINE),
      ('log_line', _LOG_LINE),
      ('turned_over_header_line', _TURNED_OVER_HEADER_LOG_LINE)]

  VERIFICATION_GRAMMAR = (
      _HEADER_LOG_LINE ^ _KNOWN_FUNCTION_LOG_LINE ^ _LOG_LINE ^
      _TURNED_OVER_HEADER_LOG_LINE)

  def _GetAction(self, action, text):
    """Parse the well known actions for easy reading.

    Args:
      action (str): the function or action called by the agent.
      text (str): text from a log line.

    Returns:
       str: a formatted string representing the known (or common) action.
           If the action is not known the original log text is returned.
    """
    # TODO: replace "x in y" checks by startswith if possible.
    if 'airportdProcessDLILEvent' in action:
      network_interface = text.split()[0]
      return 'Interface {0:s} turn up.'.format(network_interface)

    if 'doAutoJoin' in action:
      match = self._CONNECTED_RE.match(text)
      if match:
        ssid = match.group(1)[1:-1]
      else:
        ssid = 'Unknown'
      return 'Wi-Fi connected to SSID: {0:s}'.format(ssid)

    if 'processSystemPSKAssoc' in action:
      wifi_parameters = self._WIFI_PARAMETERS_RE.search(text)
      if wifi_parameters:
        ssid = wifi_parameters.group(1)
        bssid = wifi_parameters.group(2)
        security = wifi_parameters.group(3)
        if not ssid:
          ssid = 'Unknown'
        if not bssid:
          bssid = 'Unknown'
        if not security:
          security = 'Unknown'

        return (
            'New wifi configured. BSSID: {0:s}, SSID: {1:s}, '
            'Security: {2:s}.').format(bssid, ssid, security)

    return text

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    event_data = MacOSWiFiLogEventData()
    event_data.added_time = self._ParseTimeElements(time_elements_structure)
    event_data.agent = self._GetValueFromStructure(structure, 'agent')
    event_data.function = self._GetValueFromStructure(structure, 'function')
    event_data.text = self._GetStringValueFromStructure(structure, 'text')

    if key == 'known_function_log_line':
      event_data.action = self._GetAction(event_data.function, event_data.text)

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      if len(time_elements_structure) == 5:
        month_string, day_of_month, hours, minutes, seconds = (
            time_elements_structure)

        milliseconds = None
      else:
        _, month_string, day_of_month, hours, minutes, seconds, milliseconds = (
            time_elements_structure)

      month = self._GetMonthFromString(month_string)

      self._UpdateYear(month)

      relative_year = self._GetRelativeYear()

      if milliseconds is None:
        time_elements_tuple = (
            relative_year, month, day_of_month, hours, minutes, seconds)

        date_time = dfdatetime_time_elements.TimeElements(
            is_delta=True, time_elements_tuple=time_elements_tuple)

      else:
        time_elements_tuple = (
            relative_year, month, day_of_month, hours, minutes, seconds,
            milliseconds)

        date_time = dfdatetime_time_elements.TimeElementsInMilliseconds(
            is_delta=True, time_elements_tuple=time_elements_tuple)

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    self._SetEstimatedYear(parser_mediator)

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(MacOSWiFiLogTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Google log text parser plugin."""

import unittest

from dfvfs.helpers import fake_file_system_builder

from plaso.parsers import text_parser
from plaso.parsers.text_plugins import google_logging

from tests.parsers.text_plugins import test_lib


class GooglelogParserTest(test_lib.TextPluginTestCase):
  """Tests for the Google logging text parser plugin."""

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = google_logging.GoogleLogTextPlugin()

    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Log file created at: 2019/07/18 06:07:40\n'
        b'Running on machine: plasotest1\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

  def testProcess(self):
    """Tests the Process function."""
    plugin = google_logging.GoogleLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['googlelog_test.INFO'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test a single-line log entry.
    expected_event_values = {
        'data_type': 'googlelog:log',
        'file_name': 'logging_functional_test_helper.py',
        'line_number': '65',
        'message': 'This line is log level 0',
        'last_written_time': '0000-12-31T23:59:59.000002'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # Test a multi-line log entry.
    expected_event_values = {
        'data_type': 'googlelog:log',
        'message': 'Interesting Stuff\n    that spans two lines',
        'last_written_time': '0000-12-31T23:59:59.000003'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Preprocessor."""

from plaso.preprocessors import generic
from plaso.preprocessors import linux
from plaso.preprocessors import macos
from plaso.preprocessors import windows



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Android turbo database files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class AndroidTurboBatteryEvent(events.EventData):
  """Android turbo battery event data.

  Attributes:
    battery_level (int): Remaining battery level, expressed as a percentage.
    battery_saver (int): Indicates if battery saver is turn on.
    charge_type (int): Indicates that the device is charging.
    recorded_time (dfdatetime.DateTimeValues): date and time the battery event
        was recorded.
  """

  DATA_TYPE = 'android:event:battery'

  def __init__(self):
    """Initializes event data."""
    super(AndroidTurboBatteryEvent, self).__init__(data_type=self.DATA_TYPE)
    self.battery_level = None
    self.battery_saver = None
    self.charge_type = None
    self.recorded_time = None


class AndroidTurboPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Android's turbo.db database files."""

  NAME = 'android_turbo'
  DATA_FORMAT = 'Android turbo SQLite database (turbo.db) file'

  REQUIRED_STRUCTURE = {
      'battery_event': frozenset([
          'timestamp_millis', 'battery_level', 'charge_type', 'battery_saver'])}

  QUERIES = [
      ('SELECT timestamp_millis, battery_level, charge_type, battery_saver '
       'FROM battery_event',
       'ParseBatteryEventRow')]

  SCHEMAS = [{
      'android_metadata': 'CREATE TABLE android_metadata (locale TEXT)',
      'battery_event':  (
          'CREATE TABLE battery_event(timestamp_millis INTEGER PRIMARY KEY '
          'DESC, battery_level INTEGER, charge_type INTEGER, battery_saver '
          'INTEGER, timezone TEXT, place_key INTEGER, FOREIGN KEY(place_key) '
          'REFERENCES charging_places(_id))'),
      'charging_places': (
          'CREATE TABLE charging_places(_id INTEGER PRIMARY KEY, place_name '
          'TEXT, place_api_id TEXT, UNIQUE(place_api_id) ON CONFLICT IGNORE)')}]

  def ParseBatteryEventRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a row from the battery_event table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = AndroidTurboBatteryEvent()
    event_data.battery_level = self._GetRowValue(
        query_hash, row, 'battery_level')
    event_data.battery_saver = self._GetRowValue(
        query_hash, row, 'battery_saver')
    event_data.charge_type = self._GetRowValue(query_hash, row, 'charge_type')

    timestamp = self._GetRowValue(query_hash, row, 'timestamp_millis')

    event_data.recorded_time = dfdatetime_posix_time.PosixTimeInMilliseconds(
        timestamp=timestamp)

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(AndroidTurboPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Spotlight Volume configuration plist plugin."""

import unittest

from plaso.parsers.plist_plugins import spotlight_volume

from tests.parsers.plist_plugins import test_lib


class SpotlightVolumeConfigurationPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Spotlight Volume configuration plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'VolumeConfiguration.plist'

    plugin = spotlight_volume.SpotlightVolumeConfigurationPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2013-06-25T05:54:43.000000+00:00',
        'data_type': 'spotlight_volume_configuration:store',
        'partial_path': '/.MobileBackups',
        'volume_identifier': '4D4BFEB5-7FE6-4033-AAAA-AAAABBBBCCCCDDDD'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Zeitgeist activity database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import zeitgeist

from tests.parsers.sqlite_plugins import test_lib


class ZeitgeistActivityDatabasePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Zeitgeist activity database plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = zeitgeist.ZeitgeistActivityDatabasePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['activity.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 44)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'zeitgeist:activity',
        'recorded_time': '2013-10-22T08:53:19.477+00:00',
        'subject_uri': 'application://rhythmbox.desktop'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for ZSH extended_history files.

References:
  https://zsh.sourceforge.io/Doc/Release/Options.html#index-EXTENDEDHISTORY
"""

import pyparsing

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class ZshHistoryEventData(events.EventData):
  """ZSH history event data.

  Attributes:
    command (str): command that was run.
    elapsed_seconds (int): number of seconds that the command took to execute.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
  """

  DATA_TYPE = 'shell:zsh:history'

  def __init__(self):
    """Initializes event data."""
    super(ZshHistoryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.command = None
    self.elapsed_seconds = None
    self.last_written_time = None


class ZshExtendedHistoryTextPlugin(interface.TextPluginWithLineContinuation):
  """Text parser plugin for ZSH extended history files."""

  NAME = 'zsh_extended_history'
  DATA_FORMAT = 'ZSH extended history file'

  ENCODING = 'utf-8'

  _INTEGER = pyparsing.Word(pyparsing.nums).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE_START = (
      pyparsing.Literal(':') + _INTEGER.set_results_name('timestamp') +
      pyparsing.Literal(':') + _INTEGER.set_results_name('elapsed_seconds') +
      pyparsing.Literal(';'))

  _LOG_LINE = (
      _LOG_LINE_START + pyparsing.restOfLine().set_results_name('command') +
      _END_OF_LINE)

  _LINE_STRUCTURES = [('log_line', _LOG_LINE)]

  # Using a regular expression here to ensure whitespace is matched accordingly.
  VERIFICATION_GRAMMAR = (
      pyparsing.Regex(r': [0-9]+:[0-9]+;\S') + pyparsing.restOfLine() +
      _END_OF_LINE)

  def __init__(self):
    """Initializes a text parser plugin."""
    super(ZshExtendedHistoryTextPlugin, self).__init__()
    self._command_lines = None
    self._event_data = None

  def _ParseFinalize(self, parser_mediator):
    """Finalizes parsing.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
    """
    if self._event_data:
      self._event_data.command = ' '.join(self._command_lines)
      self._command_lines = []

      parser_mediator.ProduceEventData(self._event_data)
      self._event_data = None

  def _ParseLogline(self, structure):
    """Parses a log line.

    Args:
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.
    """
    timestamp = self._GetValueFromStructure(structure, 'timestamp')

    command = self._GetValueFromStructure(structure, 'command')

    event_data = ZshHistoryEventData()
    event_data.elapsed_seconds = self._GetValueFromStructure(
        structure, 'elapsed_seconds')
    event_data.last_written_time = dfdatetime_posix_time.PosixTime(
        timestamp=timestamp)

    self._event_data = event_data
    self._command_lines = [command]

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if key == '_line_continuation':
      command = structure.replace('\n', ' ').strip()
      self._command_lines.append(command)

    else:
      if self._event_data:
        self._event_data.command = ' '.join(self._command_lines)

        parser_mediator.ProduceEventData(self._event_data)

      self._ParseLogline(structure)

  def _ResetState(self):
    """Resets stored values."""
    self._command_lines = []
    self._event_data = None

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the parser.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    self._ResetState()

    return True


text_parser.TextLogParser.RegisterPlugin(ZshExtendedHistoryTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for delimiter separated values shared output module."""

import unittest

from tests.output import test_lib


class DSVOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests the delimiter separated values shared output module."""

  # TODO: add coverage for _SanitizeField
  # TODO: add coverage for SetFieldDelimiter
  # TODO: add coverage for SetFields


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS Wi-Fi log (wifi.log) files text parser plugin."""

import unittest

from plaso.parsers.text_plugins import macos_wifi

from tests.parsers.text_plugins import test_lib


class MacOSWiFiLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the MacOS MacOS Wi-Fi log (wifi.log) files text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = macos_wifi.MacOSWiFiLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['wifi.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'action': 'Interface en0 turn up.',
        'added_time': '0000-11-14T20:36:37.222+00:00',
        'agent': 'airportd[88]',
        'data_type': 'macos:wifi_log:entry',
        'function': 'airportdProcessDLILEvent',
        'text': 'en0 attached (up)'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # Check year change.
    expected_event_values = {
        'action': 'Wi-Fi connected to SSID: AndroidAP',
        'added_time': '0001-01-01T01:12:17.311+00:00',
        'agent': 'airportd[88]',
        'data_type': 'macos:wifi_log:entry',
        'function': '_doAutoJoin',
        'text': 'Already associated to “AndroidAP”. Bailing on auto-join.'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 9)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithTurnedOverLog(self):
    """Tests the Process function with a turned over log file."""
    plugin = macos_wifi.MacOSWiFiLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['wifi_turned_over.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'action': None,
        'added_time': '0000-01-02T00:10:15+00:00',
        'agent': None,
        'data_type': 'macos:wifi_log:entry',
        'function': None,
        'text': 'test-macbookpro newsyslog[50498]: logfile turned over'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'action': None,
        'added_time': '0000-01-02T00:11:02.378+00:00',
        'agent': None,
        'data_type': 'macos:wifi_log:entry',
        'function': None,
        'text': (
            '<kernel> wl0: powerChange: *** '
            'BONJOUR/MDNS OFFLOADS ARE NOT RUNNING.')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Mac OS background items plist plugin."""

import unittest

from plaso.parsers.plist_plugins import macos_background_items

from tests.parsers.plist_plugins import test_lib


class MacOSBackgroundItemsPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Mac OS background items plist parser plugin."""

  def testProcessWithBackgroundItemsBtm(self):
    """Tests the Process function with a backgrounditems.btm file."""
    plist_name = 'backgrounditems.btm'

    plugin = macos_background_items.MacOSBackgroundItemsPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:background_items:entry',
        'name': 'iTunesHelper',
        'target_creation_time': '2017-07-12T18:29:32.000000+00:00',
        'target_path': (
            '/Applications/iTunes.app/Contents/MacOS/iTunesHelper.app'),
        'volume_creation_time': '2017-10-20T07:52:27.000000+00:00',
        'volume_flags': 0x100000081,
        'volume_mount_point': '/',
        'volume_name': 'Macintosh HD'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  # TODO: add tests for version 4, 7 or 8


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS launchd plist plugin."""

import unittest

from plaso.parsers.plist_plugins import launchd

from tests.parsers.plist_plugins import test_lib


class MacOSLaunchdPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the MacOS launchd plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'launchd.plist'

    plugin = launchd.MacOSLaunchdPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:launchd:entry',
        'group_name': 'nobody',
        'name': 'com.foobar.test',
        'program': '/Test --flag arg1',
        'user_name': 'nobody'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessMinimal(self):
    """Tests the Process function."""
    plist_name = 'launchd.minimal.plist'

    plugin = launchd.MacOSLaunchdPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:launchd:entry',
        'name': 'foo',
        'program': '/usr/bin/true'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessNoProgram(self):
    """Tests the Process function."""
    plist_name = 'launchd.noprogram.plist'

    plugin = launchd.MacOSLaunchdPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:launchd:entry',
        'name': 'foo',
        'program': '/usr/bin/true'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows SetupAPI log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import setupapi

from tests.parsers.text_plugins import test_lib


class SetupAPILogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the Windows SetupAPI log text parser plugin."""

  def testProcessWithDevLog(self):
    """Tests the Process function with setupapi.dev.log."""
    plugin = setupapi.SetupAPILogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['setupapi.dev.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 194)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'setupapi:log:line',
        'end_time': '2016-10-12T03:36:30.998',
        'entry_type': (
            'Device Install (DiInstallDriver) - C:\\Windows\\System32'
            '\\DriverStore\\FileRepository\\prnms003.inf_x86_8f17aac186c70ea6'
            '\\prnms003.inf'),
        'exit_status': 'SUCCESS',
        'start_time': '2016-10-12T03:36:30.936'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 28)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithSetupLog(self):
    """Tests the Process function with setupapi.setup.log."""
    plugin = setupapi.SetupAPILogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['setupapi.setup.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 16)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'setupapi:log:line',
        'end_time': '2015-11-22T17:57:17.704',
        'entry_type': (
            'Setup Import Driver Package - C:\\Windows\\system32'
            '\\spool\\tools\\Microsoft XPS Document Writer\\prnms001.Inf'),
        'start_time': '2015-11-22T17:57:17.502'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 15)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The definitions."""


MILLISECONDS_PER_SECOND = 1000
MICROSECONDS_PER_SECOND = 1000000
MICROSECONDS_PER_MINUTE = 60000000
NANOSECONDS_PER_SECOND = 1000000000

# Characters that are considered non-printable Unicode characters.
NON_PRINTABLE_CHARACTERS = {}

# Escape C0 control characters as \x##
NON_PRINTABLE_CHARACTERS.update({
    value: f'\\x{value:02x}' for value in range(0, 0x20)})

# Escape C1 control character as \x##
NON_PRINTABLE_CHARACTERS.update({
    value: f'\\x{value:02x}' for value in range(0x7f, 0xa0)})

# Escape Unicode surrogate characters as \U########
NON_PRINTABLE_CHARACTERS.update({
    value: f'\\U{value:08x}' for value in range(0xd800, 0xe000)})

NON_PRINTABLE_CHARACTER_TRANSLATION_TABLE = str.maketrans(
    NON_PRINTABLE_CHARACTERS)

# Compression formats.
COMPRESSION_FORMAT_NONE = 'none'
COMPRESSION_FORMAT_ZLIB = 'zlib'

COMPRESSION_FORMATS = frozenset([
    COMPRESSION_FORMAT_NONE,
    COMPRESSION_FORMAT_ZLIB])

# Operating system families.
OPERATING_SYSTEM_FAMILY_LINUX = 'Linux'
OPERATING_SYSTEM_FAMILY_MACOS = 'MacOS'
OPERATING_SYSTEM_FAMILY_UNKNOWN = 'Unknown'
OPERATING_SYSTEM_FAMILY_WINDOWS_9x = 'Windows 9x/Me'
OPERATING_SYSTEM_FAMILY_WINDOWS_NT = 'Windows NT'

OPERATING_SYSTEM_FAMILIES = frozenset([
    OPERATING_SYSTEM_FAMILY_LINUX,
    OPERATING_SYSTEM_FAMILY_MACOS,
    OPERATING_SYSTEM_FAMILY_UNKNOWN,
    OPERATING_SYSTEM_FAMILY_WINDOWS_9x,
    OPERATING_SYSTEM_FAMILY_WINDOWS_NT])

# Serialization formats.
SERIALIZER_FORMAT_JSON = 'json'

SERIALIZER_FORMATS = frozenset([SERIALIZER_FORMAT_JSON])

# Source types.
SOURCE_TYPE_ARCHIVE = 'archive'

# Storage formats.
STORAGE_FORMAT_SQLITE = 'sqlite'
STORAGE_FORMAT_REDIS = 'redis'

SESSION_STORAGE_FORMATS = frozenset([
    STORAGE_FORMAT_SQLITE])

TASK_STORAGE_FORMATS = frozenset([
    STORAGE_FORMAT_SQLITE,
    STORAGE_FORMAT_REDIS])

DEFAULT_STORAGE_FORMAT = STORAGE_FORMAT_SQLITE

# Storage types.

# The session storage contains the results of one or more sessions.
# A typical session is a single run of a tool (log2timeline.py).
# The task storage contains the results of one or more tasks. Tasks
# are used to split work within a session. A typical task is a single
# run of a worker process.

STORAGE_TYPE_SESSION = 'session'
STORAGE_TYPE_TASK = 'task'

STORAGE_TYPES = frozenset([STORAGE_TYPE_SESSION, STORAGE_TYPE_TASK])

# Default worker process memory limit of 2 GiB.
DEFAULT_WORKER_MEMORY_LIMIT = 2048 * 1024 * 1024

# Consider a worker process inactive after 15 minutes of no status updates.
DEFAULT_WORKER_TIMEOUT = 15.0 * 60.0

FAILURE_MODE_EXHAUST_MEMORY = 'exhaust_memory'
FAILURE_MODE_NOT_RESPONDING = 'not_responding'
FAILURE_MODE_TERMINATED = 'terminated'
FAILURE_MODE_TIME_OUT = 'time_out'

STATUS_INDICATOR_ABORTED = 'aborted'
STATUS_INDICATOR_ANALYZING = 'analyzing'
STATUS_INDICATOR_COLLECTING = 'collecting'
STATUS_INDICATOR_COMPLETED = 'completed'
STATUS_INDICATOR_ERROR = 'error'
STATUS_INDICATOR_EXPORTING = 'exporting'
STATUS_INDICATOR_EXTRACTING = 'extracting'
STATUS_INDICATOR_FINALIZING = 'finalizing'
STATUS_INDICATOR_HASHING = 'hashing'
STATUS_INDICATOR_IDLE = 'idle'
STATUS_INDICATOR_INITIALIZED = 'initialized'
STATUS_INDICATOR_KILLED = 'killed'
STATUS_INDICATOR_MERGING = 'merging'
STATUS_INDICATOR_NOT_RESPONDING = 'not responding'
STATUS_INDICATOR_REPORTING = 'reporting'
STATUS_INDICATOR_RUNNING = 'running'
STATUS_INDICATOR_TIMELINING = 'timelining'
STATUS_INDICATOR_YARA_SCAN = 'yara scan'

ERROR_STATUS_INDICATORS = frozenset([
    STATUS_INDICATOR_ABORTED,
    STATUS_INDICATOR_ERROR,
    STATUS_INDICATOR_NOT_RESPONDING,
    STATUS_INDICATOR_KILLED])

# Time descriptions.

TIME_DESCRIPTION_ADDED = 'Added Time'
TIME_DESCRIPTION_BACKUP = 'Backup Time'
TIME_DESCRIPTION_CONNECTION_ESTABLISHED = 'Connection Established'
TIME_DESCRIPTION_CONNECTION_FAILED = 'Connection Failed'
TIME_DESCRIPTION_CREATION = 'Creation Time'
TIME_DESCRIPTION_DELETED = 'Content Deletion Time'
TIME_DESCRIPTION_DOWNGRADE = 'Downgrade Time'
TIME_DESCRIPTION_END = 'End Time'
TIME_DESCRIPTION_EXIT = 'Exit Time'
TIME_DESCRIPTION_EXPIRATION = 'Expiration Time'
TIME_DESCRIPTION_FILE_DOWNLOADED = 'File Downloaded Time'
TIME_DESCRIPTION_FIRST_ACCESS = 'First Access Time'
TIME_DESCRIPTION_FIRST_CONNECTED = 'First Connection Time'
TIME_DESCRIPTION_INSTALLATION = 'Installation Time'
TIME_DESCRIPTION_LAST_ACCESS = 'Last Access Time'
TIME_DESCRIPTION_LAST_ACTIVE = 'Last Active Time'
TIME_DESCRIPTION_LAST_CHECKED = 'Last Checked Time'
TIME_DESCRIPTION_LAST_CONNECTED = 'Last Connection Time'
TIME_DESCRIPTION_LAST_LOGIN = 'Last Login Time'
TIME_DESCRIPTION_LAST_LOGIN_ATTEMPT = 'Last Login Attempt Time'
TIME_DESCRIPTION_LAST_PASSWORD_RESET = 'Last Password Reset Time'
TIME_DESCRIPTION_LAST_PASSWORD_SET = 'Last Password Set Time'
TIME_DESCRIPTION_LAST_PRINTED = 'Last Printed Time'
TIME_DESCRIPTION_LAST_PROMPTED_USER = 'Last Prompted User'
TIME_DESCRIPTION_LAST_RESUME = 'Last Resume Time'
TIME_DESCRIPTION_LAST_RUN = 'Last Time Executed'
TIME_DESCRIPTION_LAST_SEEN = 'Last Seen Time'
TIME_DESCRIPTION_LAST_SHUTDOWN = 'Last Shutdown Time'
TIME_DESCRIPTION_LAST_UPDATED = 'Last Update Time'
TIME_DESCRIPTION_LAST_USED = 'Last Used Time'
TIME_DESCRIPTION_LAST_VISITED = 'Last Visited Time'
TIME_DESCRIPTION_LINK_TIME = 'Link Time'
TIME_DESCRIPTION_METADATA_MODIFICATION = 'Metadata Modification Time'
TIME_DESCRIPTION_MODIFICATION = 'Content Modification Time'
TIME_DESCRIPTION_PURCHASED = 'Purchased'
TIME_DESCRIPTION_RECORDED = 'Recorded Time'
TIME_DESCRIPTION_REQUEST_RECEIVED = 'Request Received Time'
TIME_DESCRIPTION_RESPONSE_SENT = 'Response Sent Time'
TIME_DESCRIPTION_SAMPLE = 'Sample Time'
TIME_DESCRIPTION_SCHEDULED_TO_END = 'Scheduled to end'
TIME_DESCRIPTION_SCHEDULED_TO_START = 'Scheduled to start'
TIME_DESCRIPTION_SENT = 'Sent Time'
TIME_DESCRIPTION_START = 'Start Time'
TIME_DESCRIPTION_UPDATE = 'Update Time'
TIME_DESCRIPTION_VALIDATION = 'Validation Time'

# Deprecated time descriptions. Do not use.
TIME_DESCRIPTION_ACCOUNT_CREATED = 'Account Created'
TIME_DESCRIPTION_CHANGE = TIME_DESCRIPTION_METADATA_MODIFICATION
TIME_DESCRIPTION_ENTRY_MODIFICATION = TIME_DESCRIPTION_METADATA_MODIFICATION
TIME_DESCRIPTION_WRITTEN = TIME_DESCRIPTION_MODIFICATION

# The timestamp does not represent a date and time value.
TIME_DESCRIPTION_NOT_A_TIME = 'Not a time'

# Note that the unknown time is used for date and time values
# of which the exact meaning is unknown and being researched.
# For most cases do not use this timestamp description.
TIME_DESCRIPTION_UNKNOWN = 'Unknown Time'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the pinfo CLI tool."""

import json
import unittest

from plaso.cli import views as cli_views
from plaso.cli import pinfo_tool
from plaso.lib import errors

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class PinfoToolTest(test_lib.CLIToolTestCase):
  """Tests for the pinfo CLI tool."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT_COMPARE_STORES = """\

************************* Events generated per parser **************************
Parser (plugin) name : Number of events
--------------------------------------------------------------------------------
            filestat : 3 (6)
               total : 3 (38)
--------------------------------------------------------------------------------

Storage files are different.
"""

  # TODO: add test for _CalculateStorageCounters.
  # TODO: add test for _CompareStores.

  def testGenerateAnalysisResultsReportAsJSON(self):
    """Tests the _GenerateAnalysisResultsReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'json'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      column_titles = ['Search engine', 'Search term', 'Number of queries']
      attribute_names = ['search_engine', 'search_term', 'number_of_queries']
      attribute_mappings = {}
      test_tool._GenerateAnalysisResultsReport(
          storage_reader, 'browser_searches', column_titles,
          'browser_search_analysis_result', attribute_names,
          attribute_mappings)

    finally:
      storage_reader.Close()

    expected_output = [
        '{"browser_searches": [', '', ']}',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateAnalysisResultsReportAsMarkdown(self):
    """Tests the _GenerateAnalysisResultsReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'markdown'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      column_titles = ['Search engine', 'Search term', 'Number of queries']
      attribute_names = ['search_engine', 'search_term', 'number_of_queries']
      attribute_mappings = {}
      test_tool._GenerateAnalysisResultsReport(
          storage_reader, 'browser_searches', column_titles,
          'browser_search_analysis_result', attribute_names,
          attribute_mappings)

    finally:
      storage_reader.Close()

    expected_output = [
        'Search engine | Search term | Number of queries',
        '--- | --- | ---',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateAnalysisResultsReportAsText(self):
    """Tests the _GenerateAnalysisResultsReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'text'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      column_titles = ['Search engine', 'Search term', 'Number of queries']
      attribute_names = ['search_engine', 'search_term', 'number_of_queries']
      attribute_mappings = {}
      test_tool._GenerateAnalysisResultsReport(
          storage_reader, 'browser_searches', column_titles,
          'browser_search_analysis_result', attribute_names,
          attribute_mappings)

    finally:
      storage_reader.Close()

    expected_output = [
        'Search engine\tSearch term\tNumber of queries',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateFileHashesReportAsJSON(self):
    """Tests the _GenerateFileHashesReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'json'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      test_tool._GenerateFileHashesReport(storage_reader)

    finally:
      storage_reader.Close()

    expected_output = [
        '{"file_hashes": [',
        ('    {"sha256_hash": '
         '"1f0105612f6ad2d225d6bd9ba631148740e312598878adcd2b74098a3dab50c4", '
         '"display_name": "OS:/tmp/test/test_data/syslog"},'),
        ('    {"sha256_hash": '
         '"1f0105612f6ad2d225d6bd9ba631148740e312598878adcd2b74098a3dab50c4", '
         '"display_name": "OS:/tmp/test/test_data/syslog"}'),
        ']}',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateFileHashesReportAsMarkdown(self):
    """Tests the _GenerateFileHashesReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'markdown'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      test_tool._GenerateFileHashesReport(storage_reader)

    finally:
      storage_reader.Close()

    expected_output = [
        'SHA256 hash | Display name',
        '--- | ---',
        ('1f0105612f6ad2d225d6bd9ba631148740e312598878adcd2b74098a3dab50c4 | '
         'OS:/tmp/test/test_data/syslog'),
        ('1f0105612f6ad2d225d6bd9ba631148740e312598878adcd2b74098a3dab50c4 | '
         'OS:/tmp/test/test_data/syslog'),
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateFileHashesReportAsText(self):
    """Tests the _GenerateFileHashesReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'text'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      test_tool._GenerateFileHashesReport(storage_reader)

    finally:
      storage_reader.Close()

    expected_output = [
        'SHA256 hash\tDisplay name',
        ('1f0105612f6ad2d225d6bd9ba631148740e312598878adcd2b74098a3dab50c4\t'
         'OS:/tmp/test/test_data/syslog'),
        ('1f0105612f6ad2d225d6bd9ba631148740e312598878adcd2b74098a3dab50c4\t'
         'OS:/tmp/test/test_data/syslog'),
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateReportEntryFormatStringAsJSON(self):
    """Tests the _GenerateReportEntryFormatString function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'json'

    attribute_names = ['search_engine', 'search_term', 'number_of_queries']

    expected_entry_format_string = (
        '    {{"search_engine": "{search_engine!s}", "search_term": '
        '"{search_term!s}", "number_of_queries": "{number_of_queries!s}"}}')

    entry_format_string = test_tool._GenerateReportEntryFormatString(
        attribute_names)
    self.assertEqual(entry_format_string, expected_entry_format_string)

  def testGenerateReportEntryFormatStringAsMarkdown(self):
    """Tests the _GenerateReportEntryFormatString function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'markdown'

    attribute_names = ['search_engine', 'search_term', 'number_of_queries']

    expected_entry_format_string = (
        '{search_engine!s} | {search_term!s} | {number_of_queries!s}\n')

    entry_format_string = test_tool._GenerateReportEntryFormatString(
        attribute_names)
    self.assertEqual(entry_format_string, expected_entry_format_string)

  def testGenerateReportEntryFormatStringAsText(self):
    """Tests the _GenerateReportEntryFormatString function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'text'

    attribute_names = ['search_engine', 'search_term', 'number_of_queries']

    expected_entry_format_string = (
        '{search_engine!s}	{search_term!s}	{number_of_queries!s}\n')

    entry_format_string = test_tool._GenerateReportEntryFormatString(
        attribute_names)
    self.assertEqual(entry_format_string, expected_entry_format_string)

  def testGenerateReportFooterAsJSON(self):
    """Tests the _GenerateReportFooter function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'json'

    test_tool._GenerateReportFooter()

    expected_output = ['', ']}', '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateReportFooterAsMarkdown(self):
    """Tests the _GenerateReportFooter function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'markdown'

    test_tool._GenerateReportFooter()

    expected_output = ['']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateReportFooterAsText(self):
    """Tests the _GenerateReportFooter function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'text'

    test_tool._GenerateReportFooter()

    expected_output = ['']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateReportHeaderAsJSON(self):
    """Tests the _GenerateReportHeader function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'json'

    column_titles = ['Search engine', 'Search term', 'Number of queries']
    test_tool._GenerateReportHeader('browser_searches', column_titles)

    expected_output = [
        '{"browser_searches": [',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateReportHeaderAsMarkdown(self):
    """Tests the _GenerateReportHeader function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'markdown'

    column_titles = ['Search engine', 'Search term', 'Number of queries']
    test_tool._GenerateReportHeader('browser_searches', column_titles)

    expected_output = [
        'Search engine | Search term | Number of queries',
        '--- | --- | ---',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateReportHeaderAsText(self):
    """Tests the _GenerateReportHeader function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'text'

    column_titles = ['Search engine', 'Search term', 'Number of queries']
    test_tool._GenerateReportHeader('browser_searches', column_titles)

    expected_output = [
        'Search engine\tSearch term\tNumber of queries',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateWinEvtProvidersReportAsJSON(self):
    """Tests the _GenerateWinEvtProvidersReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'json'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      test_tool._GenerateWinEvtProvidersReport(storage_reader)

    finally:
      storage_reader.Close()

    expected_output = [
        '{"winevt_providers": [', '', ']}',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateWinEvtProvidersReportAsMarkdown(self):
    """Tests the _GenerateWinEvtProvidersReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'markdown'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      test_tool._GenerateWinEvtProvidersReport(storage_reader)

    finally:
      storage_reader.Close()

    expected_output = [
        ('Identifier | Log source(s) | Log type(s) | Event message file(s) | '
         'Parameter message file(s)'),
        '--- | --- | --- | --- | ---',
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGenerateWinEvtProvidersReportAsText(self):
    """Tests the _GenerateWinEvtProvidersReport function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool._output_format = 'text'

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      test_tool._GenerateWinEvtProvidersReport(storage_reader)

    finally:
      storage_reader.Close()

    expected_output = [
        ('Identifier\tLog source(s)\tLog type(s)\tEvent message file(s)\t'
         'Parameter message file(s)'),
        '']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output)

  def testGetStorageReader(self):
    """Tests the _GetStorageReader function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    storage_reader = test_tool._GetStorageReader(test_file_path)
    try:
      self.assertIsNotNone(storage_reader)
    finally:
      storage_reader.Close()

    with self.assertRaises(errors.BadConfigOption):
      test_tool._GetStorageReader('bogus.plaso')

  # TODO: add test for _PrintAnalysisReportCounter.
  # TODO: add test for _PrintAnalysisReportsDetails.
  # TODO: add test for _PrintExtractionWarningsDetails.
  # TODO: add test for _PrintEventLabelsCounter.
  # TODO: add test for _PrintParsersCounter.
  # TODO: add test for _PrintPreprocessingInformation.
  # TODO: add test for _PrintRecoveryWarningsDetails.
  # TODO: add test for _PrintSessionsDetails.
  # TODO: add test for _PrintSessionsOverview.
  # TODO: add test for _PrintTasksInformation.

  def testCompareStores(self):
    """Tests the CompareStores function."""
    test_file_path1 = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path1)

    test_file_path2 = self._GetTestFilePath(['pinfo_test.plaso'])
    self._SkipIfPathNotExists(test_file_path2)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.compare_storage_file = test_file_path1
    options.storage_file = test_file_path1

    test_tool.ParseOptions(options)

    self.assertTrue(test_tool.CompareStores())

    output = output_writer.ReadOutput()
    self.assertEqual(output, 'Storage files are identical.\n')

    options = test_lib.TestOptions()
    options.compare_storage_file = test_file_path1
    options.storage_file = test_file_path2

    test_tool.ParseOptions(options)

    self.assertFalse(test_tool.CompareStores())

    output = output_writer.ReadOutput()
    self.assertEqual(output, self._EXPECTED_OUTPUT_COMPARE_STORES)

  def testParseArguments(self):
    """Tests the ParseArguments function."""
    output_writer = test_lib.TestBinaryOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    result = test_tool.ParseArguments([])
    self.assertFalse(result)

    # TODO: check output.
    # TODO: improve test coverage.

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    test_file_path = self._GetTestFilePath(['pinfo_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.storage_file = test_file_path

    test_tool.ParseOptions(options)

    options = test_lib.TestOptions()

    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    # TODO: improve test coverage.

  def testPrintStorageInformationAsJSON(self):
    """Tests the PrintStorageInformation function with JSON output format."""
    test_filename = 'pinfo_test.plaso'
    session_identifier = '0db86b5f-9176-4863-bf1e-9ac7ca632377'
    session_start_time = '2023-03-27 03:47:24.091665'

    test_file_path = self._GetTestFilePath([test_filename])
    self._SkipIfPathNotExists(test_file_path)

    options = test_lib.TestOptions()
    options.storage_file = test_file_path
    options.output_format = 'json'
    options.sections = 'events,reports,sessions,warnings'

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool.ParseOptions(options)

    test_tool.PrintStorageInformation()
    output = output_writer.ReadOutput()
    json_output = json.loads(output)

    sessions = json_output.get('sessions')
    self.assertIsNotNone(sessions)

    first_session = sessions.get('session')
    self.assertIsNotNone(first_session)

    self.assertEqual(
        first_session['identifier'], session_identifier.replace('-', ''))

    expected_start_time = shared_test_lib.CopyTimestampFromString(
        session_start_time)
    self.assertEqual(first_session['start_time'], expected_start_time)

    storage_counters = json_output.get('storage_counters')
    self.assertIsNotNone(storage_counters)

    parsers_counter = storage_counters['parsers']
    self.assertIsNotNone(parsers_counter)
    self.assertEqual(parsers_counter['total'], 3)
    self.assertEqual(parsers_counter['filestat'], 3)

  def testPrintStorageInformationAsText(self):
    """Tests the PrintStorageInformation function with text output format."""
    test_filename = 'pinfo_test.plaso'
    format_version = '20230327'
    plaso_version = '20230311'
    session_identifier = '0db86b5f-9176-4863-bf1e-9ac7ca632377'
    session_start_time = '2023-03-27T03:47:24.091665+00:00'
    session_completion_time = '2023-03-27T03:47:32.596408+00:00'

    command_line_arguments = (
        './tools/log2timeline.py --partition=all --quiet '
        '--storage-file pinfo_test.plaso test_data/tsk_volume_system.raw')

    enabled_parser_names = ', '.join([
        'android_app_usage',
        'asl_log',
        'bencode',
        'bencode/bencode_transmission',
        'bencode/bencode_utorrent',
        'binary_cookies',
        'bodyfile',
        'bsm_log',
        'chrome_cache',
        'chrome_preferences',
        'cups_ipp',
        'custom_destinations',
        'czip',
        'czip/oxml',
        'esedb',
        'esedb/file_history',
        'esedb/msie_webcache',
        'esedb/srum',
        'esedb/user_access_logging',
        'filestat',
        'firefox_cache',
        'firefox_cache2',
        'fish_history',
        'fseventsd',
        'java_idx',
        'jsonl',
        'jsonl/aws_cloudtrail_log',
        'jsonl/azure_activity_log',
        'jsonl/azure_application_gateway_access_log',
        'jsonl/docker_container_config',
        'jsonl/docker_container_log',
        'jsonl/docker_layer_config',
        'jsonl/gcp_log',
        'jsonl/ios_application_privacy',
        'jsonl/microsoft_audit_log',
        'lnk',
        'locate_database',
        'mac_keychain',
        'mcafee_protection',
        'mft',
        'msiecf',
        'networkminer_fileinfo',
        'olecf',
        'olecf/olecf_automatic_destinations',
        'olecf/olecf_default',
        'olecf/olecf_document_summary',
        'olecf/olecf_summary',
        'opera_global',
        'opera_typed_history',
        'pe',
        'plist',
        'plist/airport',
        'plist/apple_id',
        'plist/ios_carplay',
        'plist/ipod_device',
        'plist/launchd_plist',
        'plist/macos_bluetooth',
        'plist/macos_software_update',
        'plist/macosx_install_history',
        'plist/macuser',
        'plist/plist_default',
        'plist/safari_downloads',
        'plist/safari_history',
        'plist/spotlight',
        'plist/spotlight_volume',
        'plist/time_machine',
        'pls_recall',
        'prefetch',
        'recycle_bin',
        'recycle_bin_info2',
        'rplog',
        'spotlight_storedb',
        'sqlite',
        'sqlite/android_calls',
        'sqlite/android_sms',
        'sqlite/android_webview',
        'sqlite/android_webviewcache',
        'sqlite/appusage',
        'sqlite/chrome_17_cookies',
        'sqlite/chrome_27_history',
        'sqlite/chrome_66_cookies',
        'sqlite/chrome_8_history',
        'sqlite/chrome_autofill',
        'sqlite/chrome_extension_activity',
        'sqlite/dropbox',
        'sqlite/firefox_cookies',
        'sqlite/firefox_downloads',
        'sqlite/firefox_history',
        'sqlite/google_drive',
        'sqlite/hangouts_messages',
        'sqlite/imessage',
        'sqlite/ios_netusage',
        'sqlite/ios_powerlog',
        'sqlite/ios_screentime',
        'sqlite/kik_ios',
        'sqlite/kodi',
        'sqlite/ls_quarantine',
        'sqlite/mac_document_versions',
        'sqlite/mac_knowledgec',
        'sqlite/mac_notes',
        'sqlite/mac_notificationcenter',
        'sqlite/mackeeper_cache',
        'sqlite/macostcc',
        'sqlite/safari_historydb',
        'sqlite/skype',
        'sqlite/tango_android_profile',
        'sqlite/tango_android_tc',
        'sqlite/twitter_android',
        'sqlite/twitter_ios',
        'sqlite/windows_eventtranscript',
        'sqlite/windows_timeline',
        'sqlite/zeitgeist',
        'symantec_scanlog',
        'systemd_journal',
        'text',
        'text/android_logcat',
        'text/apache_access',
        'text/apt_history',
        'text/aws_elb_access',
        'text/bash_history',
        'text/confluence_access',
        'text/dpkg',
        'text/gdrive_synclog',
        'text/googlelog',
        'text/ios_lockdownd',
        'text/ios_logd',
        'text/ios_sysdiag_log',
        'text/mac_appfirewall_log',
        'text/mac_securityd',
        'text/mac_wifi',
        'text/popularity_contest',
        'text/postgresql',
        'text/santa',
        'text/sccm',
        'text/selinux',
        'text/setupapi',
        'text/skydrive_log_v1',
        'text/skydrive_log_v2',
        'text/snort_fastlog',
        'text/sophos_av',
        'text/syslog',
        'text/syslog_traditional',
        'text/viminfo',
        'text/vsftpd',
        'text/winfirewall',
        'text/winiis',
        'text/xchatlog',
        'text/xchatscrollback',
        'text/zsh_extended_history',
        'trendmicro_url',
        'trendmicro_vd',
        'usnjrnl',
        'utmp',
        'utmpx',
        'windefender_history',
        'winevt',
        'winevtx',
        'winjob',
        'winpca_db0',
        'winpca_dic',
        'winreg',
        'winreg/amcache',
        'winreg/appcompatcache',
        'winreg/bagmru',
        'winreg/bam',
        'winreg/ccleaner',
        'winreg/explorer_mountpoints2',
        'winreg/explorer_programscache',
        'winreg/microsoft_office_mru',
        'winreg/microsoft_outlook_mru',
        'winreg/mrulist_shell_item_list',
        'winreg/mrulist_string',
        'winreg/mrulistex_shell_item_list',
        'winreg/mrulistex_string',
        'winreg/mrulistex_string_and_shell_item',
        'winreg/mrulistex_string_and_shell_item_list',
        'winreg/msie_zone',
        'winreg/mstsc_rdp',
        'winreg/mstsc_rdp_mru',
        'winreg/network_drives',
        'winreg/networks',
        'winreg/userassist',
        'winreg/windows_boot_execute',
        'winreg/windows_boot_verify',
        'winreg/windows_run',
        'winreg/windows_sam_users',
        'winreg/windows_services',
        'winreg/windows_shutdown',
        'winreg/windows_task_cache',
        'winreg/windows_timezone',
        'winreg/windows_typed_urls',
        'winreg/windows_usb_devices',
        'winreg/windows_usbstor_devices',
        'winreg/windows_version',
        'winreg/winlogon',
        'winreg/winrar_mru',
        'winreg/winreg_default'])

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')

    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI,
        title='Plaso Storage Information')
    table_view.AddRow(['Filename', test_filename])
    table_view.AddRow(['Format version', format_version])
    table_view.AddRow(['Serialization format', 'json'])
    table_view.Write(output_writer)

    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI, title='Sessions')
    table_view.AddRow([session_identifier, session_start_time])
    table_view.Write(output_writer)

    title = 'Session: {0!s}'.format(session_identifier)
    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI, title=title)
    table_view.AddRow(['Start time', session_start_time])
    table_view.AddRow(['Completion time', session_completion_time])
    table_view.AddRow(['Product name', 'plaso'])
    table_view.AddRow(['Product version', plaso_version])
    table_view.AddRow(['Command line arguments', command_line_arguments])
    table_view.AddRow(['Parser filter expression', 'N/A'])
    table_view.AddRow(['Enabled parser and plugins', enabled_parser_names])
    table_view.AddRow(['Preferred encoding', 'UTF-8'])
    table_view.AddRow(['Preferred time zone', 'UTC'])
    table_view.AddRow(['Debug mode', 'False'])
    table_view.AddRow(['Artifact filters', 'N/A'])
    table_view.AddRow(['Filter file', 'N/A'])
    table_view.Write(output_writer)

    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI,
        column_names=['Parser (plugin) name', 'Number of events'],
        title='Events generated per parser')
    table_view.AddRow(['filestat', '3'])
    table_view.AddRow(['Total', '3'])
    table_view.Write(output_writer)

    expected_output = output_writer.ReadOutput()

    expected_output = (
        '{0:s}'
        '\n'
        'No events labels stored.\n'
        '\n'
        'No warnings stored.\n'
        '\n'
        'No analysis reports stored.\n'
        '\n').format(expected_output)

    test_file_path = self._GetTestFilePath([test_filename])
    self._SkipIfPathNotExists(test_file_path)

    options = test_lib.TestOptions()
    options.storage_file = test_file_path
    options.output_format = 'text'
    options.sections = 'events,reports,sessions,warnings'

    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_tool.ParseOptions(options)

    test_tool.PrintStorageInformation()

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split('\n'), expected_output.split('\n'))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Mac OS startup item plist plugin."""

import unittest

from plaso.parsers.plist_plugins import macos_startup_item

from tests.parsers.plist_plugins import test_lib


class MacOSStartupItemPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Mac OS startup item plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'StartupParameters.plist'

    plugin = macos_startup_item.MacOSStartupItemPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Note that the startup item event data has no date and time attribute.
    expected_event_values = {
        'data_type': 'macos:startup_item:entry',
        'description': 'My Awesome Service',
        'order_preference': 'None',
        'provides': ['The best service ever'],
        'uses': ['Network', 'SystemLog']}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Mac OS login window plist plugin."""
import unittest

from plaso.parsers.plist_plugins import macos_login_window

from tests.parsers.plist_plugins import test_lib


class MacOSLoginWindowPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Mac OS login window plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'loginwindow.plist'

    plugin = macos_login_window.MacOSLoginWindowPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:login_window:managed_login_item',
        'is_hidden': True,
        'path': '/Applications/SafeConnect.app'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'macos:login_window:entry',
        'login_hook': '/Users/matthew/login.sh',
        'logout_hook': '/Users/matthew/logout.sh'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""JSON attribute container serializer."""

import binascii
import codecs
import collections
import json

from acstore.containers import interface as containers_interface
from acstore.helpers import json_serializer
from acstore.helpers import schema as schema_helper

from dfdatetime import interface as dfdatetime_interface
from dfdatetime import serializer as dfdatetime_serializer

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import path_spec as dfvfs_path_spec
from dfvfs.path import factory as dfvfs_path_spec_factory

# The following import is needed to make sure TSKTime is registered with
# the dfDateTime factory.
from dfvfs.vfs import tsk_file_entry  # pylint: disable=unused-import

from plaso.storage import serializers


schema_helper.SchemaHelper.RegisterDataTypes({
    'dfdatetime.DateTimeValues': {
        'json': serializers.JSONDateTimeAttributeSerializer()},
    'dfvfs.PathSpec': {
        'json': serializers.JSONPathSpecAttributeSerializer()},
    'List[int]': {
        'json': serializers.JSONValueListAttributeSerializer()},
    'List[str]': {
        'json': serializers.JSONValueListAttributeSerializer()}})


class JSONAttributeContainerSerializer(
    json_serializer.AttributeContainerJSONSerializer):
  """JSON attribute container serializer."""

  _convert_json_to_value = {}

  @classmethod
  def _ConvertAttributeContainerToJSON(cls, attribute_container):
    """Converts an attribute container object into a JSON dictionary.

    The resulting dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'AttributeContainer'
        '__container_type__': ...
        ...
    }

    Here '__type__' indicates the object base type. In this case
    'AttributeContainer'.

    '__container_type__' indicates the container type and rest of the elements
    of the dictionary that make up the attributes of the container.

    Args:
      attribute_container (AttributeContainer): attribute container.

    Returns:
      dict[str, object]: JSON serialized objects.
    """
    if attribute_container.CONTAINER_TYPE not in (
        'event_data', 'system_configuration'):
      return cls.ConvertAttributeContainerToJSON(attribute_container)

    json_dict = {
        '__type__': 'AttributeContainer',
        '__container_type__': attribute_container.CONTAINER_TYPE}

    for attribute_name, attribute_value in attribute_container.GetAttributes():
      json_dict[attribute_name] = cls._ConvertValueToJSON(attribute_value)

    return json_dict

  @classmethod
  def _ConvertBytesToJSON(cls, bytes_value):
    """Converts a bytes value into a JSON dictionary.

    Args:
      bytes_value (bytes): a bytes value.

    Returns:
      dict: JSON serialized object of a bytes value.
    """
    encoded_value = binascii.b2a_qp(bytes_value)
    encoded_value = codecs.decode(encoded_value, 'ascii')
    return {
        '__encoding__': 'base16',
        '__type__': 'bytes',
        'stream': encoded_value}

  @classmethod
  def _ConvertCollectionsCounterToJSON(cls, counter_value):
    """Converts a collections.Counter object into a JSON dictionary.

    The resulting dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'collections.Counter'
        ...
    }

    Here '__type__' indicates the object base type. In this case
    'collections.Counter'. The rest of the elements of the dictionary make up
    the collections.Counter object attributes.

    Args:
      counter_value (collections.Counter): a collections.Counter value.

    Returns:
      dict[str, object]: JSON serialized objects.
    """
    json_dict = {'__type__': 'collections.Counter'}
    for attribute_name, attribute_value in counter_value.items():
      if attribute_value:
        if isinstance(attribute_value, bytes):
          attribute_value = cls._ConvertBytesToJSON(attribute_value)

        json_dict[attribute_name] = attribute_value

    return json_dict

  @classmethod
  def _ConvertJSONToAttributeContainer(cls, json_dict):
    """Converts a JSON dictionary into an attribute container.

    The dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'AttributeContainer'
        '__container_type__': ...
        ...
    }

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      AttributeContainer: an attribute container.

    Raises:
      ValueError: if the container type or attribute type of an event data
          attribute container is not supported.
    """
    # Use __container_type__ to indicate the attribute container type.
    container_type = json_dict.get('__container_type__', None)

    if container_type not in ('event_data', 'system_configuration'):
      return cls.ConvertJSONToAttributeContainer(json_dict)

    attribute_container = cls._CONTAINERS_MANAGER.CreateAttributeContainer(
        container_type)

    for attribute_name, attribute_value in json_dict.items():
      if isinstance(attribute_value, dict):
        attribute_value = cls._ConvertJSONToValue(attribute_value)

      elif isinstance(attribute_value, list):
        attribute_value = cls._ConvertListToValue(attribute_value)

      if isinstance(attribute_value, bytes):
        raise ValueError((
            f'Event data attribute value: {attribute_name:s} of type bytes '
            f'is not supported.'))

      if isinstance(attribute_value, dict):
        raise ValueError((
            f'Event data attribute value: {attribute_name:s} of type dict '
            f'is not supported.'))

      setattr(attribute_container, attribute_name, attribute_value)

    return attribute_container

  @classmethod
  def _ConvertJSONToBytes(cls, json_dict):
    """Converts a JSON dictionary into a bytes value.

    The dictionary of the JSON serialized objects consists of:
    {
        '__encoding__': encoding of stream, typically base16.
        '__type__': 'bytes'
        'stream': encoded bytes value.
    }

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      bytes: a bytes value.

    Raises:
      ValueError: if the encoding is not supported.
    """
    encoding = json_dict.get('__encoding__', None) or 'base16'
    if encoding != 'base16':
      raise ValueError(f'Unsupported byte stream encoding: {encoding:s}')

    return binascii.a2b_qp(json_dict['stream'])

  @classmethod
  def _ConvertJSONToCollectionsCounter(cls, json_dict):
    """Converts a JSON dictionary into a collections.Counter.

    The dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'collections.Counter'
        ...
    }

    Here '__type__' indicates the object base type. In this case
    'collections.Counter'. The rest of the elements of the dictionary make up
    the collections.Counter object attributes.

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      collections.Counter: a collections.Counter value.
    """
    collections_counter = collections.Counter()

    for key, value in json_dict.items():
      if key != '__type__':
        collections_counter[key] = value

    return collections_counter

  @classmethod
  def _ConvertJSONToTuple(cls, json_dict):
    """Converts a JSON dictionary into a tuple value.

    The dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'tuple'
        'values': elements of the tuple.
    }

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      tuple: a tuple value.
    """
    return tuple(cls._ConvertListToValue(json_dict['values']))

  # Pylint is confused by the formatting of the return type.
  # pylint: disable=missing-return-type-doc
  @classmethod
  def _ConvertJSONToValue(cls, json_dict):
    """Converts a JSON dictionary into a value.

    The dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'AttributeContainer'
        '__container_type__': ...
        ...
    }

    Here '__type__' indicates the object base type. In this case
    'AttributeContainer'.

    '__container_type__' indicates the attribute container type.

    The rest of the elements of the dictionary make up the attributes.

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      AttributeContainer|dict|list|tuple: deserialized object.

    Raises:
      ValueError: if the type of a JSON dictionary value is not supported.
    """
    if not cls._convert_json_to_value:
      cls._convert_json_to_value = {
        'AttributeContainer': cls._ConvertJSONToAttributeContainer,
        'bytes': cls._ConvertJSONToBytes,
        'collections.Counter': cls._ConvertJSONToCollectionsCounter,
        'DateTimeValues': (
            dfdatetime_serializer.Serializer.ConvertDictToDateTimeValues),
        'PathSpec': cls._ConvertJSONToPathSpec,
        'tuple': cls._ConvertJSONToTuple}

    json_dict_type = json_dict.get('__type__', None)
    if not json_dict_type:
      return json_dict

    convert_function = cls._convert_json_to_value.get(json_dict_type, None)
    if not convert_function:
      raise ValueError(f'Unsupported JSON dictionary type: {json_dict_type:s}')

    return convert_function(json_dict)

  @classmethod
  def _ConvertJSONToPathSpec(cls, json_dict):
    """Converts a JSON dictionary into a path specification object.

    The dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'PathSpec'
        'type_indicator': 'OS'
        'parent': { ... }
        ...
    }

    Here '__type__' indicates the object base type. In this case this should
    be 'PathSpec'. The rest of the elements of the dictionary make up the
    path specification object properties.

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      dfvfs.PathSpec: path specification.
    """
    type_indicator = json_dict.get('type_indicator', None)
    if type_indicator:
      del json_dict['type_indicator']

    if 'parent' in json_dict:
      json_dict['parent'] = cls._ConvertJSONToPathSpec(json_dict['parent'])

    # Remove the class type from the JSON dictionary since we cannot pass it.
    del json_dict['__type__']

    path_spec = dfvfs_path_spec_factory.Factory.NewPathSpec(
        type_indicator, **json_dict)

    if type_indicator == dfvfs_definitions.TYPE_INDICATOR_OS:
      # dfvfs.OSPathSpec() will change the location to an absolute path
      # here we want to preserve the original location.
      path_spec.location = json_dict.get('location', None)

    return path_spec

  @classmethod
  def _ConvertListToJSON(cls, value):
    """Converts a list value into a JSON dictionary.

    Args:
      value (list): a list value.

    Returns:
      list: JSON serialized object of a list value.
    """
    return [cls._ConvertValueToJSON(element) for element in value]

  @classmethod
  def _ConvertListToValue(cls, json_list):
    """Converts a JSON list into an object.

    Args:
      json_list (list[object]): JSON serialized objects.

    Returns:
      list[object]: a deserialized list.
    """
    list_value = []
    for json_list_element in json_list:
      if isinstance(json_list_element, dict):
        list_value.append(cls._ConvertJSONToValue(json_list_element))

      elif isinstance(json_list_element, list):
        list_value.append(cls._ConvertListToValue(json_list_element))

      else:
        list_value.append(json_list_element)

    return list_value

  @classmethod
  def _ConvertTupleToJSON(cls, value):
    """Converts a tuple value into a JSON dictionary.

    The dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'tuple'
        'values': elements of the tuple.
    }

    Args:
      value (tuple): a tuple value.

    Returns:
      dict: JSON serialized object of a tuple value.
    """
    return {
      '__type__': 'tuple',
        'values': cls._ConvertListToJSON(value)}

  @classmethod
  def _ConvertPathSpecToJSON(cls, path_spec_object):
    """Converts a path specification object into a JSON dictionary.

    The resulting dictionary of the JSON serialized objects consists of:
    {
        '__type__': 'PathSpec'
        'type_indicator': 'OS'
        'parent': { ... }
        ...
    }

    Here '__type__' indicates the object base type. In this case 'PathSpec'.
    The rest of the elements of the dictionary make up the path specification
    object properties. The supported property names are defined in
    path_spec_factory.Factory.PROPERTY_NAMES. Note that this method is called
    recursively for every path specification object and creates a dict of
    dicts in the process.

    Args:
      path_spec_object (dfvfs.PathSpec): path specification.

    Returns:
      dict[str, object]: JSON serialized objects.

    Raises:
      TypeError: if not an instance of dfvfs.PathSpec.
    """
    if not isinstance(path_spec_object, dfvfs_path_spec.PathSpec):
      raise TypeError

    json_dict = {'__type__': 'PathSpec'}
    for property_name in dfvfs_path_spec_factory.Factory.PROPERTY_NAMES:
      property_value = getattr(path_spec_object, property_name, None)
      if property_value is not None:
        json_dict[property_name] = property_value

    if path_spec_object.HasParent():
      json_dict['parent'] = cls._ConvertPathSpecToJSON(path_spec_object.parent)

    json_dict['type_indicator'] = path_spec_object.type_indicator
    location = getattr(path_spec_object, 'location', None)
    if location:
      json_dict['location'] = location

    return json_dict

  # Pylint is confused by the formatting of the return type.
  # pylint: disable=missing-return-type-doc
  @classmethod
  def _ConvertValueToJSON(cls, attribute_value):
    """Converts a value into a JSON dictionary.

    Args:
      attribute_value (object): an attribute value.

    Returns:
      dict|list: The JSON serialized object which can be a dictionary or a list.
    """
    convert_function = None

    if isinstance(attribute_value, bytes):
      convert_function = cls._ConvertBytesToJSON

    elif isinstance(attribute_value, list):
      convert_function = cls._ConvertListToJSON

    elif isinstance(attribute_value, tuple):
      convert_function = cls._ConvertTupleToJSON

    elif isinstance(attribute_value, collections.Counter):
      convert_function = cls._ConvertCollectionsCounterToJSON

    elif isinstance(attribute_value, dfdatetime_interface.DateTimeValues):
      convert_function = (
          dfdatetime_serializer.Serializer.ConvertDateTimeValuesToDict)

    elif isinstance(attribute_value, dfvfs_path_spec.PathSpec):
      convert_function = cls._ConvertPathSpecToJSON

    elif isinstance(attribute_value, containers_interface.AttributeContainer):
      convert_function = cls.ConvertAttributeContainerToJSON

    if convert_function:
      attribute_value = convert_function(attribute_value)

    return attribute_value

  @classmethod
  def ReadSerialized(cls, json_string):  # pylint: disable=arguments-renamed
    """Reads an attribute container from serialized form.

    Args:
      json_string (str): JSON serialized attribute container.

    Returns:
      AttributeContainer: attribute container or None.
    """
    if json_string:
      json_dict = json.loads(json_string)
      return cls.ReadSerializedDict(json_dict)

    return None

  @classmethod
  def ReadSerializedDict(cls, json_dict):
    """Reads an attribute container from serialized dictionary form.

    Args:
      json_dict (dict[str, object]): JSON serialized objects.

    Returns:
      AttributeContainer: attribute container or None.

    Raises:
      TypeError: if the serialized dictionary does not contain an
          AttributeContainer.
    """
    if not json_dict:
      return None

    if isinstance(json_dict, list):
      return json_dict

    json_object = cls._ConvertJSONToValue(json_dict)

    if not isinstance(json_object, (
        containers_interface.AttributeContainer,
        dfdatetime_interface.DateTimeValues,
        dfvfs_path_spec.PathSpec)):
      json_object_type = type(json_object)
      raise TypeError(f'{json_object_type!s} is not a supported type.')

    return json_object

  @classmethod
  def WriteSerialized(cls, attribute_container):
    """Writes an attribute container to serialized form.

    Args:
      attribute_container (AttributeContainer): attribute container.

    Returns:
      str: A JSON string containing the serialized form.
    """
    json_dict = cls.WriteSerializedDict(attribute_container)
    return json.dumps(json_dict)

  @classmethod
  def WriteSerializedDict(cls, attribute_container):
    """Writes an attribute container to serialized form.

    Args:
      attribute_container (AttributeContainer): attribute container.

    Returns:
      dict[str, object]: JSON serialized objects.
    """
    if isinstance(attribute_container, list):
      return attribute_container

    if isinstance(attribute_container, dfdatetime_interface.DateTimeValues):
      return dfdatetime_serializer.Serializer.ConvertDateTimeValuesToDict(
          attribute_container)

    if isinstance(attribute_container, dfvfs_path_spec.PathSpec):
      return cls._ConvertPathSpecToJSON(attribute_container)

    return cls._ConvertAttributeContainerToJSON(attribute_container)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Trend Micro AV Log parser."""

import unittest

from plaso.parsers import trendmicroav

from tests.parsers import test_lib


class TrendMicroUnitTest(test_lib.ParserTestCase):
  """Tests for the Trend Micro AV Log parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = trendmicroav.OfficeScanVirusDetectionParser()
    storage_writer = self._ParseFile(['pccnt35.log'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'action': 10,
        'data_type': 'av:trendmicro:scan',
        'filename': 'eicar.com_.gstmp',
        'path': 'C:\\temp\\',
        'scan_type': 1,
        'threat': 'Eicar_test_1',
        'written_time': '2018-01-30T14:45:32+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # The third and last log entry has been edited to match the older,
    # documented format for log lines (without a POSIX timestamp).
    expected_event_values = {
        'action': 10,
        'data_type': 'av:trendmicro:scan',
        'filename': 'eicar.com_.gstmp',
        'path': 'C:\\temp\\',
        'scan_type': 1,
        'threat': 'Eicar_test_1',
        'written_time': '2018-01-30T14:46:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

  def testWebReputationParse(self):
    """Tests the Parse function."""
    parser = trendmicroav.OfficeScanWebReputationParser()
    storage_writer = self._ParseFile(['OfcUrlf.log'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application_name': 'C:\\Users\\user\\Downloads\\wget.exe',
        'block_mode': 1,
        'credibility_rating': 1,
        'credibility_score': 49,
        'data_type': 'av:trendmicro:webrep',
        'group_code': '4E',
        'group_name': 'Malware Accomplice',
        'policy_identifier': 1,
        'threshold': 0,
        'url': 'http://www.eicar.org/download/eicar.com',
        'written_time': '2018-01-23T13:17:02+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the TeamViewer log parser."""

import unittest

from plaso.parsers.text_plugins import teamviewer

from tests.parsers.text_plugins import test_lib


class TeamViewerApplicationLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the TeamViewer application log text file parser plugin."""

  _TEST_BODY = """\
Start:              2024/02/16 06:05:19.548 (UTC-8:00)
Version:            15.50.5 (64-bit)
Version short hash: b23ae08066e
ID:                 0
Loglevel:           Info
License:            0
IC:                 -2100253538
CPU:                AMD64 Family 23 Model 113 Stepping 0, AuthenticAMD
CPU extensions:     e9
OS:                 Win_10.0.17763_W (64-bit)
IP:                 10.0.2.15
MID:                0x080027e6e559_1d44cc67230d0d5_3030843078
MIDv:               0
Proxy-Settings:     Type=1 IP= User=
IE                  11.1790.17763.0
AppPath:            C:\\Program Files\\TeamViewer\\TeamViewer_Service.exe
UserAccount:        SYSTEM"""

  def testProcess(self):
    """Tests the Process function."""
    plugin = teamviewer.TeamViewerApplicationLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['teamviewer', 'TeamViewer15_Logfile.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1013)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values_multi_line = {
        'body': self._TEST_BODY,
        'process_identifier': 2136,
        'recorded_time': '2024-02-16T06:05:19.561+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 17)
    self.CheckEventData(event_data, expected_event_values_multi_line)

    expected_event_values_single_line = {
        'body': 'MDV2: IsDeviceManagementEnabled: mdv2: 1 management: 1',
        'process_identifier': 5276,
        'recorded_time': '2024-02-16T06:18:36.370+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1012)
    self.CheckEventData(event_data, expected_event_values_single_line)


class TeamViewerConnectionsIncomingLogTextPluginTest(
    test_lib.TextPluginTestCase):
  """Tests for the TeamViewer connections_incoming.txt log parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = teamviewer.TeamViewerConnectionsIncomingLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['teamviewer', 'connections_incoming.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'activity_type': 'RemoteControl',
        'connection_identifier': '{b3a4df33-d027-44d5-b50f-4e61115494d4}',
        'data_type': 'teamviewer:connections_incoming:entry',
        'display_name': 'TestUserRedacted',
        'end_time': '2024-02-16T14:18:36+00:00',
        'local_account': 'IEUser',
        'source_identifier': 1660360496,
        'start_time': '2024-02-16T14:16:32+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class TeamViewerConnectionsOutgoingLogTextPluginTest(
    test_lib.TextPluginTestCase):
  """Tests for the TeamViewer Connections.txt log file parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = teamviewer.TeamViewerConnectionsOutgoingLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['teamviewer', 'Connections.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'activity_type': 'RemoteControl',
        'connection_identifier': '{b35110b2-7dc5-4bb9-8d54-ca80953f65f8}',
        'data_type': 'teamviewer:connections_outgoing:entry',
        'destination_identifier': 1660360496,
        'end_time': '2024-02-20T13:11:52+00:00',
        'local_account': 'IEUser',
        'start_time': '2024-02-20T13:10:33+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()
